// Copyright (C) 2018  Nexedi SA and Contributors.
//                     Kirill Smelkov <kirr@nexedi.com>
//
// This program is free software: you can Use, Study, Modify and Redistribute
// it under the terms of the GNU General Public License version 3, or (at your
// option) any later version, as published by the Free Software Foundation.
//
// You can also Link and Combine this program with other software covered by
// the terms of any of the Free Software licenses or any of the Open Source
// Initiative approved licenses and Convey the resulting work. Corresponding
// source of such a combination shall include the source code for all other
// software used.
//
// This program is distributed WITHOUT ANY WARRANTY; without even the implied
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// See COPYING file for full licensing terms.
// See https://www.nexedi.com/licensing for rationale and options.

package main

//go:generate ./testdata/zblk_test_gen.py

import (
	"context"
	"encoding/binary"
	"testing"

	"lab.nexedi.com/kirr/go123/exc"
	"lab.nexedi.com/kirr/neo/go/transaction"
	"lab.nexedi.com/kirr/neo/go/zodb"
	_ "lab.nexedi.com/kirr/neo/go/zodb/wks"

	"github.com/stretchr/testify/require"
)

func TestZBlk(t *testing.T) {
	X := exc.Raiseif
	assert := require.New(t)
	ctx := context.Background()
	stor, err := zodb.OpenStorage(ctx, "testdata/zblk.fs", &zodb.OpenOptions{ReadOnly: true}); X(err)
	db := zodb.NewDB(stor)

	txn, ctx := transaction.New(ctx)
	defer txn.Abort()

	conn, err := db.Open(ctx, &zodb.ConnOptions{});	X(err)

	// XXX close db/stor

	xz0, err := conn.Get(ctx, z0_oid);	X(err)
	xz1, err := conn.Get(ctx, z1_oid);	X(err)
	xzf, err := conn.Get(ctx, zf_oid);	X(err)

	z0, ok := xz0.(*ZBlk0)
	if !ok {
		t.Fatalf("z0: want ZBlk0; got %T", xz0)
	}

	z1, ok := xz1.(*ZBlk1)
	if !ok {
		t.Fatalf("z1: want ZBlk1; got %T", xz1)
	}

	zf, ok := xzf.(*ZBigFile)
	if !ok {
		t.Fatalf("zf: want ZBigFile; got %T", xzf)
	}

	xactivate := func(obj zodb.IPersistent) {
		err := obj.PActivate(ctx)
		if err != nil {
			t.Fatal(err)
		}
	}

	// data that must be stored in ZBlk*
	data := make([]byte, blksize)
	for i := uint32(0); i < blksize / 4; i++ {
		binary.BigEndian.PutUint32(data[i*4:], i)
	}

	z0Data, err := z0.LoadBlkData(ctx);	X(err)
	assert.Equal(z0Data, data, "ZBlk0 data wrong")

	z1Data, err := z1.LoadBlkData(ctx);	X(err)
	assert.Equal(z1Data, data, "ZBlk0 data wrong")



	// XXX
	_, _, _ = z0, z1, zf
	_ = xactivate

/*
	xactivate(zf)
	if zf.blksize != blksize {
		t.Fatalf("zf: blksize=%d;  want %d", zf.blksize, blksize)
	}

	// XXX zf.blktab.Get(0) -> z0
	// XXX zf.blktab.Get(1) -> z1
*/

}
